//
//	mondo solitaire Games - Copyright  2005, glenn andreas, All Rights Reserved
//
/*global solitaireGame, slShuffleDeck, slClearAllPiles */
/*global slCardTableWidth, slCardTableHeight, slCardWidth, slCardHeight */

function klondikeGame(variantName) {
	this.inheritsFrom = solitaireGame;
	this.inheritsFrom(variantName);
	// finally, build the defaults
	this.buildPiles = function() {
		if (this.variantName == "Klondike (1 Card)") {
			this.buildBottomDeckAndWaste(1); // flip three at a time
		} else {
			this.buildBottomDeckAndWaste(3); // flip three at a time
		}
		this.buildHorizFoundation(0, slCardTableWidth, 10, 4);
		this.buildHorizTableaux(0, slCardTableWidth, 20 + slCardHeight, 7);
	}
	this.newGame = function() {
		slShuffleDeck();
		// clear all the piles
		slClearAllPiles();
		// add shuffled cards - build tableux
		for (var i=0;i<7;i++) {
			this.dealToPile(i+1,this.tableauxPiles[i],null);
		}
		this.remainderToDeck();
		this.deckPile.clickRoutine(null,0,0);
	}
	this.ruleStats = 'memm';
	this.ruleVariants = {"Klondike (1 Card)" : "Only one card is turned over from the stock at a time"};
}
klondikeGame.name = "Klondike";
klondikeGame.variants = new Array("Klondike (1 Card)");

AddGame(klondikeGame);


function yukonGame(variantName) {
	this.inheritsFrom = solitaireGame;
	this.inheritsFrom(variantName);
	// finally, build the defaults
	this.buildPiles = function() {
		// only foundation & tableaux
		this.buildHorizFoundation(0, slCardTableWidth, 10, 4);
		this.buildHorizTableaux(0, slCardTableWidth, 20 + slCardHeight, 7);
		for (var i=0;i<7;i++) {
			// can drop same suit up or down
			if (this.variantName == "Alaska") {
				this.tableauxPiles[i].dropRule = slDropRuleSameSuitUpOrDown;
			}
			this.tableauxPiles[i].dragPileRule = true; // drag any number of face up
			this.tableauxPiles[i].cardYStagger = 10; // smush in
		}
	}
	this.newGame = function() {
		slShuffleDeck();
		// clear all the piles
		slClearAllPiles();
		// add shuffled cards - build tableux
		for (var i=0;i<7;i++) {
			this.dealToPile(i+1,this.tableauxPiles[i],null);
		}
		// and deal rest face up
		for (var i=1;i<7;i++) {
			this.dealToPile(4, this.tableauxPiles[i],true);
		}
	}
	this.ruleStats ="mmmh";
	this.ruleVariants = {'Alaska':'Tableaux builds using the same suit, ascending or descending.'};
}
yukonGame.name = "Yukon";
yukonGame.variants = new Array("Alaska");
AddGame(yukonGame);

function acesUpGame(variantName) {
	this.inheritsFrom = solitaireGame;
	this.inheritsFrom(variantName);
	this.buildPiles = function() {
		this.buildHorizFoundation(0, slCardTableWidth, 10, 4);
		this.buildHorizTableaux(0, slCardTableWidth, 20 + slCardHeight, 7);
		for (var i=0;i<7;i++) {
			this.tableauxPiles[i].cardYStagger = 10; // can get long, so smush
			this.tableauxPiles[i].downCardYStagger = 4;
		}
		this.deckPile = slCreatePile(10,slCardTableHeight - slCardHeight,slCardWidth,slCardHeight,slDeckPile,"Deck");
		this.deckPile.cardXStagger = 8;
		this.deckPile.dealToPiles = this.tableauxPiles;
	}
	this.newGame = function() {
		slShuffleDeck();
		slClearAllPiles();
		for (var i=0;i<7;i++) {
			this.dealToPile(2,this.tableauxPiles[i],false); // two down
			this.dealToPile(1,this.tableauxPiles[i], true); // one up
		}
		this.remainderToDeck();
	}
	this.ruleStats ="qecl";
}
acesUpGame.name = "Aces Up";
acesUpGame.ruleSimilar = "Non-Builder";
AddGame(acesUpGame);

function canfieldGame(variantName) {
	this.inheritsFrom = solitaireGame;
	this.inheritsFrom(variantName);
	this.buildPiles = function() {
		this.buildBottomDeckAndWaste(3); // flip three at a time
		this.reservePiles = new Array(slCreatePile(10,10,slCardWidth,200, slReservePile, "Reserve"));
		this.reservePiles[0].cardYStagger = 18;
		this.wastePile.cardXStagger = slCardWidth / 5; // and squeze the waste pile a bit
		// and build these shifted over by the reserve
		this.buildHorizTableaux(slCardWidth + 10, slCardTableWidth, 20 + slCardHeight, 4);
		for (var i=0;i<4;i++) {
			this.tableauxPiles[i].dropRule = slDropRuleAltColorDownAnyBlank; // can put anything on a blank
			this.tableauxPiles[i].autoFillFromPile = this.reservePiles[0]; // and automatically pull from reserve when blank
		}
		this.buildHorizFoundation(slCardWidth + 10, slCardTableWidth, 10, 4);
	}
	this.newGame = function() {
		slShuffleDeck();
		// clear all the piles
		slClearAllPiles();
		// tableaux
		for (var i=0;i<4;i++) {
			this.dealToPile(1,this.tableauxPiles[i],true);
		}
		// and reserve
		this.dealToPile(13,this.reservePiles[0],true);
		this.remainderToDeck();
	}
	this.ruleStats = "mecl";
}
canfieldGame.name = "Canfield";
AddGame(canfieldGame);




function freeCellGame(variantName) {
	this.inheritsFrom = solitaireGame;
	this.inheritsFrom(variantName);
	this.buildPiles = function() {
		this.buildHorizFoundation(slCardTableWidth / 2, slCardTableWidth - 10, 10, 4);
		this.buildHorizPiles(10, slCardTableWidth / 2, 10, 4, 3, slFreeCellPile, "FreeCell", slCardHeight, this.freeCellPiles);
		this.buildHorizTableaux(0, slCardTableWidth, 20 + slCardHeight, 8);
		for (var i=0;i<8;i++) {
			this.tableauxPiles[i].dragPileRule = null; // can't drag more than one
			if (this.variantName != "Kings Only") {
				this.tableauxPiles[i].blankPileTopCard = "*X"; // can put anything in a blank spot
			}
		}
	}
	this.newGame = function() {
		slShuffleDeck();
		slClearAllPiles();
		for (var i=0;i<8;i++) {
			if (i < 4) {
				this.dealToPile(7, this.tableauxPiles[i], true);
			} else {
				this.dealToPile(6, this.tableauxPiles[i], true);
			}
		}
	}
	this.ruleStats = "mmsh";
	this.ruleVariants = {'Kings Only':'Only a king can be played on a blank Tableaux spot.'};
}
freeCellGame.name = "FreeCell";
freeCellGame.variants = new Array("Kings Only");
AddGame(freeCellGame);

function seaHavenGame(variantName) {
	this.inheritsFrom = solitaireGame;
	this.inheritsFrom(variantName);
	this.buildPiles = function() {
		this.buildHorizFoundation(slCardTableWidth / 2, slCardTableWidth - 10, 10, 4);
		this.buildHorizPiles(10, slCardTableWidth / 2, 10, 4, 3, slFreeCellPile, "FreeCell", slCardHeight, this.freeCellPiles);
		this.buildHorizTableaux(0, slCardTableWidth, 20 + slCardHeight, 10);
		for (var i=0;i<10;i++) {
			this.tableauxPiles[i].dragPileRule = null; // can't drag more than one
			if (this.variantName == "Sea Saw") {
				this.tableauxPiles[i].dropRule = slDropRuleSameSuitUpOrDown;
			} else {
				this.tableauxPiles[i].dropRule = slDropRuleSameSuitDown; // down, in suit
			}
		}
	}
	this.newGame = function() {
		slShuffleDeck();
		slClearAllPiles();
		for (var i=0;i<10;i++) {
			this.dealToPile(5, this.tableauxPiles[i], true);
		}
		this.dealToPile(1, this.freeCellPiles[1], true);
		this.dealToPile(1, this.freeCellPiles[2], true);
	}
	this.ruleStats = "mmsh";
	this.ruleVariants = {'Sea Saw':'Tableaux can be played either up or down in the same suit.'};
}
seaHavenGame.name = "Sea Haven Tower";
seaHavenGame.variants = new Array("Sea Saw");
seaHavenGame.ruleSimilar = "FreeCell";
AddGame(seaHavenGame);


function kingAlbertGame(variantName) {
	this.inheritsFrom = solitaireGame;
	this.inheritsFrom(variantName);
	this.reservePiles = new Array();
	this.buildPiles = function() {
		this.buildHorizFoundation(slCardTableWidth / 2, slCardTableWidth - 10, 10, 4);
		this.buildHorizPiles(10, slCardTableWidth / 2, 10, 7, -slCardWidth/2 + 10, slReservePile, "Reserves", slCardHeight, this.reservePiles);
		this.buildHorizTableaux(0, slCardTableWidth, 20 + slCardHeight, 9);
		for (var i=0;i<9;i++) {
			this.tableauxPiles[i].dragPileRule = null; // can't drag more than one
			this.tableauxPiles[i].blankPileTopCard = "*X"; // can put anything in a blank spot
		}
	}
	this.newGame = function() {
		slShuffleDeck();
		slClearAllPiles();
		for (var i=0;i<7;i++) {
			this.dealToPile(1, this.reservePiles[i], true);
		}
		for (var i=0;i<9;i++) {
			this.dealToPile(i+1, this.tableauxPiles[i], true);
		}
	}
	this.ruleStats = "mmSl";
}
kingAlbertGame.name = "King Albert";
kingAlbertGame.ruleSimilar = "Klondike";
AddGame(kingAlbertGame);


function fanGame(variantName) {
	this.inheritsFrom = solitaireGame;
	this.inheritsFrom(variantName);
	this.buildPiles = function() {
		this.buildHorizFoundation(10, slCardTableWidth - 10, 10, 4);
		// split into two rows
		this.buildHorizTableaux(0, slCardTableWidth, 20 + slCardHeight, 9);
		this.buildHorizTableaux(0, slCardTableWidth,  (slCardTableHeight - (20 + slCardHeight)) / 2 + 20 + slCardHeight, 9);
		for (var i=0;i<18;i++) {
			this.tableauxPiles[i].dropRule = slDropRuleSameSuitUpOrDown; // up or down, in suit
		}
	}
	this.newGame = function() {
		slShuffleDeck();
		slClearAllPiles();
		for (var i=0;i<18;i++) {
			if (i < 16) {
				this.dealToPile(3, this.tableauxPiles[i], true);
			} else {
				this.dealToPile(2, this.tableauxPiles[i], true);
			}
		}
	}
	this.ruleStats = "mmsl";
}
fanGame.name = "Fan";

AddGame(fanGame);


function golfHolePile(div) {
	this.inheritsFrom = slPileBase;
	this.inheritsFrom(div);
	this.dropRule = slDropAnyUpOrDown;
	this.canDragTop = false;
	this.bestDropRank = 100;
	this.isComplete = function () {
		return this.pile.length + this.deckPile.pile.length == 52;
	}
}

function golfGame(variantName)	{
	this.inheritsFrom = solitaireGame;
	this.inheritsFrom(variantName);
	this.holePile = null;
	this.buildPiles = function() {
		this.buildHorizTableaux(0, slCardTableWidth, 20, 7);
		for (var i=0;i<7;i++) {
			if (this.variantName == "Swamp Drainer") {
				// allow reorder
				this.tableauxPiles[i].cardYStagger = slCardHeight / 4;
			} else {
				this.tableauxPiles[i].cardYStagger = slCardHeight / 2;
				this.tableauxPiles[i].dropRule = false;
				this.tableauxPiles[i].showsEmptyImage = false;
			}
		}
		this.deckPile = slCreatePile(slCardWidth,slCardTableHeight - slCardHeight,slCardWidth + 60,slCardHeight,slDeckPile,"Deck");
		this.deckPile.flipToWasteCount = 1;
		this.deckPile.cardYStagger = 0;
		this.deckPile.cardXStagger = 10; // stagger to the left
		this.deckPile.dealLimit = 0;
		this.deckPile.showsEmptyImage = false;
		this.deckPile.currentEmptyImage = null;
		this.deckPile.isComplete = function() { return true; } // we're complete regardless

		this.holePile = slCreatePile(slCardTableWidth - slCardWidth * 2,slCardTableHeight - slCardHeight,slCardWidth, slCardHeight, golfHolePile,"Hole");
		if (this.variantName == "Golf (Wrap)") {
			this.holePile.dropRule = slDropAnyUpOrDownWrap;
		}
		this.holePile.deckPile = this.deckPile;
		this.deckPile.wastePile = this.holePile;
	}
	this.newGame = function() {
		slShuffleDeck();
		slClearAllPiles();
		for (var i=0;i<7;i++) {
			if (this.variantName == "Swamp Drainer") {
				this.dealToPile(6, this.tableauxPiles[i], true);
			} else {
				this.dealToPile(5, this.tableauxPiles[i], true);
			}
		}
		this.remainderToDeck();
		this.deckPile.clickRoutine(null,0,0);
	}
	this.ruleStats = "qecl";
	this.ruleGoal = "Sink all the cards into the Waste pile";
	this.ruleVariants = {
		'Swamp Drainer':'An extra row of cards is dealt in the tableaux, but you can reorder cards in the tableuax',
		'Golf (Wrap)':'Slightly higher chance of winning because you can wrap between Ace and King in building.'
		};
}
golfGame.name = "Golf";
golfGame.ruleSimilar = "Simple";
golfGame.variants = new Array("Golf (Wrap)", "Swamp Drainer");
AddGame(golfGame);


function beleagueredCastleGame(variantName) {
	this.inheritsFrom = solitaireGame;
	this.inheritsFrom(variantName);
	this.buildPiles = function() {
		this.buildVertTableaux(10,slCardTableHeight - 10, 10, 4);
		this.buildVertTableaux(10,slCardTableHeight - 10, slCardTableWidth / 2 + slCardWidth / 2 + 10, 4);
		for (var i=0;i<8;i++) {
			if (this.variantName == "Less Beleaguered Castle") {
				this.tableauxPiles[i].dropRule = slDropAnyUpOrDown; // drop anything descending
			} else {
				this.tableauxPiles[i].dropRule = slDropAnyDown; // drop anything descending
			}
			this.tableauxPiles[i].blankPileTopCard = "*X"; // can put anything
			this.tableauxPiles[i].dragPileRule = false; // can't drag multiple
			// build horizontally
			this.tableauxPiles[i].cardXStagger = 10;
			this.tableauxPiles[i].cardYStagger = 0;
		}
		this.buildVertFoundation(10, slCardTableHeight - 10, slCardTableWidth / 2 - slCardWidth/2, 4);
	}
	this.newGame = function() {
		slShuffleDeck();
		slClearAllPiles();
		if (this.variantName == "Streets and Alleys") {
			// don't start with the aces up
		} else {
			slRemoveFromDeck(new Array("AC","AH","AS","AD")); // remove the aces
			this.acesToPiles(this.foundationPiles); // put aces on foundation
		}
		for (var i=0;i<8;i++) {
			if (this.variantName == "Streets and Alleys" && i < 4) {
				this.dealToPile(7, this.tableauxPiles[i], true);
			} else {
				this.dealToPile(6, this.tableauxPiles[i], true);
			}
		}
	}
	this.ruleStats = 'mmsl';
	this.ruleVariants = {
		"Less Beleaguered Castle":"Tableaux can be built ascending or descending, instead of just decending.",
		"Streets and Alleys":"Aces are left in the deck rather than starting in the foundation."
	}
}
beleagueredCastleGame.name = "Beleaguered Castle";
beleagueredCastleGame.variants = new Array("Less Beleaguered Castle", "Streets and Alleys");
AddGame(beleagueredCastleGame);


function fortressGame(variantName) {
	this.inheritsFrom = solitaireGame;
	this.inheritsFrom(variantName);
	this.buildPiles = function() {
		this.buildVertTableaux(10,slCardTableHeight - 10, 10, 5);
		this.buildVertTableaux(10,slCardTableHeight - 10, slCardTableWidth / 2 + slCardWidth / 2 + 10, 5);
		for (var i=0;i<10;i++) {
			this.tableauxPiles[i].dropRule = slDropRuleSameSuitUpOrDown;
			this.tableauxPiles[i].blankPileTopCard = "*X"; // can put anything
			this.tableauxPiles[i].dragPileRule = false; // can't drag multiple
			// build horizontally
			this.tableauxPiles[i].cardXStagger = 10;
			this.tableauxPiles[i].cardYStagger = 0;
		}
		this.buildVertFoundation(10, slCardTableHeight - 10, slCardTableWidth / 2 - slCardWidth/2, 4);
	}
	this.newGame = function() {
		slShuffleDeck();
		slClearAllPiles();
		for (var i=0;i<10;i++) {
			this.dealToPile(5, this.tableauxPiles[i], true);
		}
		// and the last two
		this.dealToPile(1, this.tableauxPiles[0], true);
		this.dealToPile(1, this.tableauxPiles[5], true);
	}
	this.ruleStats = 'mesl';
}
fortressGame.name = "Fortress";
fortressGame.ruleSimilar = "Beleaguered Castle";
AddGame(fortressGame);




function fourtyThievesGame(variantName) {
	this.inheritsFrom = solitaireGame;
	this.inheritsFrom(variantName);
	this.numberOfDecks = 2;
	this.cardSize = mediumCards;
	this.buildPiles = function() {
		this.buildHorizFoundation(10, slCardTableWidth - 10, 10, 8);
		var t = 10;
		if (variantName == "Limited") t = 12;
		if (variantName == "Maria") t = 9;
		this.buildHorizTableaux(0, slCardTableWidth, slCardHeight + 20, t);
		for (var i=0;i<t;i++) {
			this.tableauxPiles[i].dragPileRule = false; // can't drag multiple
			this.tableauxPiles[i].blankPileTopCard = "*X"; // can put anything
			if (variantName == "Maria") {
				this.tableauxPiles[i].dropRule = slDropRuleAltColorDownAnyBlank;
			} else {
				this.tableauxPiles[i].dropRule = slDropRuleSameSuitDown; // drop anything descending
			}
		}
		this.buildBottomDeckAndWaste(1); // flip three at a time
	}
	this.newGame = function() {
		slShuffleDeck();
		slClearAllPiles();
		if (variantName == "Lucas") {
			slRemoveFromDeck(new Array("AC","AH","AS","AD")); // remove the aces
			this.acesToPiles(this.foundationPiles); // put aces on foundation
		}
		if (variantName == "Limited") {
			for (var i=0;i<12;i++) {
				this.dealToPile(3, this.tableauxPiles[i], true);
			}
		} else if (variantName == "Maria") {
			for (var i=0;i<9;i++) {
				this.dealToPile(4, this.tableauxPiles[i], true);
			}
		} else {
			for (var i=0;i<10;i++) {
				this.dealToPile(4, this.tableauxPiles[i], true);
			}
		}
		this.remainderToDeck();
		this.deckPile.clickRoutine(null,0,0);
	}
	this.ruleStats = 'lesl';
	this.ruleVariants = {
		'Limited':'Tableaux has twelve columns of three cards each.',
		'Lucas':'Aces start out in the foundation.',
		'Maria':'Tableaux has nine columns of four cards each, and are build with descending alternate colors.'
	}
}
fourtyThievesGame.name = "Fourty Thieves";
fourtyThievesGame.variants = new Array("Limited", "Lucas", "Maria");
AddGame(fourtyThievesGame);

function spiderGame(variantName) {
	this.inheritsFrom = solitaireGame;
	this.inheritsFrom(variantName);
	this.numberOfDecks = 2;
	this.cardSize = mediumCards;
	this.buildPiles = function() {
		this.buildHorizFoundation(10, slCardTableWidth - 10, 10, 8);
		for (var i=0;i<8;i++) {
			this.foundationPiles[i].dropRule = slDropRuleFullSuit;
			this.foundationPiles[i].canDragTop = false; // can't play off
		}
		this.buildHorizTableaux(0, slCardTableWidth, slCardHeight + 20, 10);
		for (var i=0;i<10;i++) {
			this.tableauxPiles[i].blankPileTopCard = "*X"; // can put anything
			this.tableauxPiles[i].dropRule = slDropAnyDown;
			this.tableauxPiles[i].dragPileRule = slDragRuleSameSuitDown; // only drag if suit matches
			this.tableauxPiles[i].cardYStagger = 9;
			this.tableauxPiles[i].downCardYStagger = 6; // smush in
		}
		this.deckPile = slCreatePile(10,slCardTableHeight - slCardHeight,slCardWidth,slCardHeight,slDeckPile,"Deck");
		this.deckPile.cardXStagger = 7;

		this.deckPile.dealToPiles = this.tableauxPiles; // deck deals to all
		this.deckPile.dealLimit = 0; // no redeals
	}
	this.newGame = function() {
		slShuffleDeck();
		slClearAllPiles();
		for (var i=0;i<10;i++) {
			if	(i<4) { // first get an extra card
				this.dealToPile(5, this.tableauxPiles[i], null);
			} else {
				this.dealToPile(4, this.tableauxPiles[i], null);
			}
		}
		this.remainderToDeck();
	}
	this.ruleGoal = "Remove all the suits to the foundations from the tableaux, once complete from King down to Ace";
	this.ruleStats = 'meSl';
}
spiderGame.name = "Spider";
AddGame(spiderGame);

function spideretteGame(variantName) {
	this.inheritsFrom = solitaireGame;
	this.inheritsFrom(variantName);
	this.numberOfDecks = 1;
	this.buildPiles = function() {
		this.buildHorizFoundation(10, slCardTableWidth - 10, 10, 4);
		for (var i=0;i<4;i++) {
			this.foundationPiles[i].dropRule = slDropRuleFullSuit;
			this.foundationPiles[i].canDragTop = false; // can't play off
		}
		this.buildHorizTableaux(0, slCardTableWidth, slCardHeight + 20, 7);
		for (var i=0;i<7;i++) {
			this.tableauxPiles[i].blankPileTopCard = "*X"; // can put anything
			this.tableauxPiles[i].dropRule = slDropAnyDown;
			this.tableauxPiles[i].dragPileRule = slDragRuleSameSuitDown; // only drag if suit matches
			this.tableauxPiles[i].cardYStagger = 14;
			this.tableauxPiles[i].downCardYStagger = 7; // smush in
		}
		this.deckPile = slCreatePile(10,slCardTableHeight - slCardHeight,slCardWidth,slCardHeight,slDeckPile,"Deck");
		this.deckPile.cardXStagger = 8;
		this.deckPile.dealToPiles = this.tableauxPiles; // deck deals to all
		this.deckPile.dealLimit = 0; // no redeals
	}
	this.newGame = function() {
		slShuffleDeck();
		slClearAllPiles();
		for (var i=0;i<7;i++) {
			if (this.variantName == "Will o' the Wisp") {
				this.dealToPile(3,this.tableauxPiles[i],null);
			} else {
				this.dealToPile(i+1,this.tableauxPiles[i],null);
			}
		}
		this.remainderToDeck();
	}
	this.ruleGoal = "Remove all the suits to the foundations from the tableaux, once complete from King down to Ace";
	this.ruleStats = 'meSL';
	this.ruleVariants = {
			"Will o' the Wisp":'Tableaux has seven piles of three cards each.'
		}
}
spideretteGame.name = "Spiderette";
spideretteGame.ruleSimilar = "Spider";
spideretteGame.variants = new Array("Will o' the Wisp");
AddGame(spideretteGame);

function westcliffGame(variantName) {
	this.inheritsFrom = solitaireGame;
	this.inheritsFrom(variantName);
	this.numberOfDecks = 1;
	this.cardSize = mediumCards;
	this.buildPiles = function() {
		this.buildHorizFoundation(10, slCardTableWidth - 10, 10, 4);
		this.buildHorizTableaux(0, slCardTableWidth, slCardHeight + 20, 10);
		for (var i=0;i<10;i++) {
			this.tableauxPiles[i].blankPileTopCard = "*X"; // can put anything
		}
		this.buildBottomDeckAndWaste(1); // flip three at a time
		this.deckPile.dealLimit = 0; // no redeals
	}
	this.newGame = function() {
		slShuffleDeck();
		slClearAllPiles();
		for (var i=0;i<10;i++) {
			this.dealToPile(3, this.tableauxPiles[i], null);
		}
		this.remainderToDeck();
		this.deckPile.clickRoutine(null,0,0);
	}
	this.ruleStats = 'mecH';
}
westcliffGame.name = "Westcliff";
westcliffGame.ruleSimilar = "Klondike";
AddGame(westcliffGame);



function scorpionGame(variantName) {
	this.inheritsFrom = solitaireGame;
	this.inheritsFrom(variantName);
	this.numberOfDecks = 1;
	this.buildPiles = function() {
		this.buildHorizFoundation(10, slCardTableWidth - 10, 10, 4);
		for (var i=0;i<4;i++) {
			this.foundationPiles[i].dropRule = slDropRuleFullSuit;
			this.foundationPiles[i].canDragTop = false; // can't play off
		}
		this.buildHorizTableaux(0, slCardTableWidth, slCardHeight + 20, 7);
		for (var i=0;i<7;i++) {
			this.tableauxPiles[i].dropRule = slDropRuleSameSuitDown; // down in same suit
			this.tableauxPiles[i].dragPileRule = true; // drag any number of face up
			this.tableauxPiles[i].cardYStagger = 10; // smush in
		}
		this.deckPile = slCreatePile(10,slCardTableHeight - slCardHeight,slCardWidth,slCardHeight,slDeckPile,"Deck");
		this.deckPile.cardXStagger = 8;
		this.deckPile.dealToPiles = this.tableauxPiles;
	}
	this.newGame = function() {
		slShuffleDeck();
		slClearAllPiles();
		for (var i=0;i<7;i++) {
			if (i<4) {
				// 3 down, 4 up
				this.dealToPile(3, this.tableauxPiles[i], false);
				this.dealToPile(4, this.tableauxPiles[i], true);
			} else {
				this.dealToPile(7, this.tableauxPiles[i], true);
			}
		}
		this.remainderToDeck(); // last three here
	}
	this.ruleStats = "mesL";
}
scorpionGame.name = "Scorpion";
scorpionGame.ruleSimilar = "Spider";
AddGame(scorpionGame);


function blockadeGame(variantName) {
	this.inheritsFrom = solitaireGame;
	this.inheritsFrom(variantName);
	this.numberOfDecks = 2;
	this.cardSize = mediumCards;
	this.buildPiles = function() {
		this.buildHorizFoundation(10, slCardTableWidth - 10, 10, 8);
		this.buildHorizTableaux(0, slCardTableWidth, slCardHeight + 20, 12);
		for (var i=0;i<12;i++) {
			this.tableauxPiles[i].blankPileTopCard = "*X";
			this.tableauxPiles[i].dropRule = slDropRuleSameSuitDown; // down in same suit
			this.tableauxPiles[i].dragPileRule = slDragRuleSameSuitDown;
		}
		this.deckPile = slCreatePile(10,slCardTableHeight - slCardHeight,slCardWidth,slCardHeight,slDeckPileFillEmptyFirst,"Deck");
		this.deckPile.cardXStagger = 4;
		this.deckPile.dealToPiles = this.tableauxPiles;
	}
	this.newGame = function() {
		slShuffleDeck();
		slClearAllPiles();
		for (var i=0;i<12;i++) {
			this.dealToPile(1, this.tableauxPiles[i], true);
		}
		this.remainderToDeck(); // last three here
	}
	this.ruleStats = "lmch";
}
blockadeGame.name = "Blockade";
AddGame(blockadeGame);


function harpGame(variantName) {
	this.inheritsFrom = solitaireGame;
	this.inheritsFrom(variantName);
	this.numberOfDecks = 2;
	this.cardSize = mediumCards;
	this.buildPiles = function() {
		this.buildBottomDeckAndWaste(1); // flip three at a time
		this.buildHorizFoundation(10, slCardTableWidth - 10, 10, 8);
		this.buildHorizTableaux(0, slCardTableWidth, slCardHeight + 20, 9);
		for (var i=0;i<9;i++) {
			if (this.variantName == "Suited Harp") {
				this.tableauxPiles[i].blankPileTopCard = "*X";
				this.tableauxPiles[i].dropRule = slDropRuleSameSuitDown;
				this.tableauxPiles[i].dragPileRule = slDragRuleSameSuitDown;
			} else {
				// special drag rule - can drag king pile as a whole
				this.tableauxPiles[i].dragPileRule = slDragRuleSingleOrKingPile;
			}
		}
	}
	this.newGame = function() {
		slShuffleDeck();
		// clear all the piles
		slClearAllPiles();
		// add shuffled cards - build tableux
		for (var i=0;i<9;i++) {
			this.dealToPile(i+1,this.tableauxPiles[i],null);
		}
		this.remainderToDeck();
		this.deckPile.clickRoutine(null,0,0);
	}
	this.ruleStats = "lecl";
	//this.ruleVariants = {"Suited Harp" : "Tableaux is build by descending suit (and can be moved as pile of the the same)."};
}
harpGame.name = "Harp";
harpGame.ruleSimilar = "Klondike";
//harpGame.variants	 = new Array("Suited Harp");
AddGame(harpGame);


function fourteensGame(variantName) {
	this.inheritsFrom = solitaireGame;
	this.inheritsFrom(variantName);
	this.buildPiles = function() {
		var p = slCombinePairs14Pile;
		if (this.variantName == "Thirteens") {
			p = slCombinePairs13Pile;
		}
		this.buildVertPiles(10,slCardTableHeight - 20 - slCardHeight, 10, 3, 6, p, "Tableux", 800, this.tableauxPiles);
		this.buildVertPiles(10,slCardTableHeight - 20 - slCardHeight, (slCardTableWidth - 40) / 4 + slCardWidth / 4 + 10, 3, 6, p, "Tableux", 800, this.tableauxPiles);
		this.buildVertPiles(10,slCardTableHeight - 20 - slCardHeight, (slCardTableWidth - 40) * 2 / 4 + slCardWidth * 2 / 4 + 10, 3, 6, p, "Tableux", 800, this.tableauxPiles);
		this.buildVertPiles(10,slCardTableHeight - 20 - slCardHeight, (slCardTableWidth - 40) * 3 / 4 + slCardWidth * 3 / 4 + 10, 3, 6, p, "Tableux", 800, this.tableauxPiles);
		for (var i=0;i<12;i++) {
			this.tableauxPiles[i].cardXStagger = 10;
			this.tableauxPiles[i].cardYStagger = 0;
		}
		if (this.variantName == "Thirteens") {
			this.wastePile = slCreatePile(10,slCardTableHeight - slCardHeight,slCardTableWidth - 20,slCardHeight, slDiscardPileKings,"Waste");
		} else {
			this.wastePile = slCreatePile(10,slCardTableHeight - slCardHeight,slCardTableWidth - 20,slCardHeight, slDiscardPile,"Waste");
		}
	}
	this.newGame = function() {
		slShuffleDeck();
		slClearAllPiles();
		for (var i=0;i<12;i++) {
			if (i % 3 == 0) {
				// 3 down, 4 up
				this.dealToPile(5, this.tableauxPiles[i], true);
			} else {
				this.dealToPile(4, this.tableauxPiles[i], true);
			}
		}
	}
	this.ruleGoal = "Combine pairs of cards to total 14, which are removed from play - remove all to win.";
	this.ruleVariants = {"Thirteens" : "Combine pairs of cards to total 13, or discard kings."};
	this.ruleStats = "qesH";
}
fourteensGame.name = "Fourteens";
fourteensGame.variants = new Array("Thirteens");
AddGame(fourteensGame);

function tripeaksGame(variantName) {
	this.inheritsFrom = solitaireGame;
	this.inheritsFrom(variantName);
	this.buildPiles = function() {
		var pattern = [
		"...#.....#.....#...",
		"..#.#...#.#...#.#..",
		".#.#.#.#.#.#.#.#.#.",
		"#.#.#.#.#.#.#.#.#.#"];
		this.buildOverlapTableaux(10, 10 + 4 * slCardHeight / 2, 4, slCardTableWidth - 8, pattern, slStaggerOverlapPile);
		this.deckPile = slCreatePile(10,slCardTableHeight - slCardHeight,slCardWidth * 2,slCardHeight,slDeckPile,"Deck");
		this.deckPile.cardXStagger = 2;
		this.foundationPiles = new Array(slCreatePile(20 + slCardWidth * 2,slCardTableHeight - slCardHeight,slCardTableWidth - 20 - slCardWidth * 2,slCardHeight,slFoundationPile,"Foundation"));
		this.foundationPiles[0].canDragTop = false;
		this.foundationPiles[0].pileLimit = 0; // all the cards go here
		this.foundationPiles[0].dropRule = slDropAnyUpOrDown;
		this.foundationPiles[0].cardXStagger = 2;
		this.foundationPiles[0].blankPileTopCard = "--";
		this.deckPile.wastePile = this.foundationPiles[0];
		this.deckPile.dealLimit = 0;
	}
	this.newGame = function() {
		slShuffleDeck();
		slClearAllPiles();
		for (var i=0;i<28;i++) {
			this.dealToPile(1, this.tableauxPiles[i], true);
		}
		this.remainderToDeck();
		this.deckPile.clickRoutine(null,0,0);
	}
	this.ruleGoal = "Build the foundation up or down to remove all the cards from the tri-peaks.";
	this.ruleStats = "qech";
}
tripeaksGame.name = "Tri-Peaks";
AddGame(tripeaksGame);

function blackHoleGame(variantName) {
	this.inheritsFrom = solitaireGame;
	this.inheritsFrom(variantName);
	this.buildPiles = function() {
		this.buildHorizPiles(slCardTableWidth/2 - 3 * slCardWidth - 25,slCardTableWidth/2 + 3 * slCardWidth + 25, 10, 6, 10, slTableauxPile, "Tableux", slCardHeight + 40, this.tableauxPiles);
		this.buildHorizPiles(slCardTableWidth/2 - 4 * slCardWidth - 25,slCardTableWidth/2 - slCardWidth - 15, 10 + slCardHeight + 40, 3, 10, slTableauxPile, "Tableux", slCardHeight + 40, this.tableauxPiles);
		this.buildHorizPiles(slCardTableWidth/2 + slCardWidth + 15,slCardTableWidth/2 + 4 * slCardWidth + 25, 10 + slCardHeight + 40, 3, 10, slTableauxPile, "Tableux", slCardHeight + 40, this.tableauxPiles);
		this.buildHorizPiles(slCardTableWidth/2 - 2.5 * slCardWidth - 25,slCardTableWidth/2 + 2.5 * slCardWidth + 25, 10 + slCardHeight * 2 + 80, 5, 10, slTableauxPile, "Tableux", slCardHeight + 40, this.tableauxPiles);
		this.foundationPiles = new Array(slCreatePile(slCardTableWidth/2 - slCardWidth/2,10 + slCardHeight + 40,slCardTableWidth,slCardHeight,slFoundationPile,"Foundation"));
		this.foundationPiles[0].canDragTop = false;
		this.foundationPiles[0].pileLimit = 0; // all the cards go here
		this.foundationPiles[0].dropRule = slDropAnyUpOrDownWrap;
		this.foundationPiles[0].blankPileTopCard = "--";
		for (var i=0;i<17;i++) {
			this.tableauxPiles[i].dropRule = false;
			this.tableauxPiles[i].dragPileRule = false;
		}
	}
	this.newGame  = function() {
		slShuffleDeck();
		slClearAllPiles();
		slRemoveFromDeck(new Array("AS")); // remove the aces
		this.foundationPiles[0].addToTop(slBuildCard(0,0,'AS',true));
		for (var i=0;i<17;i++) {
			this.dealToPile(3, this.tableauxPiles[i], true);
		}
	}
	this.ruleGoal = "Build the center foundation up or down to remove all the cards.";
	this.ruleStats = "qmml";
}
blackHoleGame.name = "Black Hole";
AddGame(blackHoleGame);


function eliminatorGame(variantName) {
	this.inheritsFrom = solitaireGame;
	this.inheritsFrom(variantName);
	this.buildPiles = function() {
		this.buildVertTableaux(20 + slCardHeight,slCardTableHeight, 10, 4);
		this.buildHorizFoundation(10, slCardTableWidth - 10, 10, 6);
		for (var i=0;i<6;i++) {
			this.foundationPiles[i].canDragTop = false;
			this.foundationPiles[i].pileLimit = 0; // all the cards could go here
			this.foundationPiles[i].dropRule = slDropAnyUpOrDown;
			this.foundationPiles[i].blankPileTopCard = "*X";
			this.foundationPiles[i].isComplete = function() { return true; } // we're complete when tableaux is empty
		}
		for (var i=0;i<4;i++) {
			this.tableauxPiles[i].dropRule = false;
			this.tableauxPiles[i].dragPileRule = false;
			this.tableauxPiles[i].cardXStagger = (slCardTableWidth - 20 - slCardWidth) / 12;
			this.tableauxPiles[i].cardYStagger = 0;
		}
	}
	this.newGame  = function() {
		slShuffleDeck();
		slClearAllPiles();
		for (var i=0;i<4;i++) {
			this.dealToPile(13, this.tableauxPiles[i], true);
		}
	}
	this.ruleGoal = "Build all the cards in any of six foundations, up or down in any suit, to remove all the cards from the tableaux.";
	this.ruleStats = "qmml";
}
eliminatorGame.name = "Eliminator";
AddGame(eliminatorGame);


function flowerGardenGame(variantName) {
	this.inheritsFrom = solitaireGame;
	this.inheritsFrom(variantName);
	this.buildPiles = function() {
		this.buildHorizFoundation(slCardTableWidth - 4 * slCardWidth - 30, slCardTableWidth - 10, 10, 4);
		var columns = 7;
		if (this.variantName == "Flower Garden 66") {
			columns = 6;
		}
		this.buildHorizTableaux(0, slCardTableWidth, slCardHeight + 20, columns);
		for (var i=0;i<columns;i++) {
			this.tableauxPiles[i].blankPileTopCard = "*X"; // can put anything
			this.tableauxPiles[i].dropRule = slDropAnyDown;
			this.tableauxPiles[i].dragPileRule = null; // only drag if suit matches
		}
		this.deckPile = slCreatePile(10,10,slCardTableWidth - 4 * slCardWidth - 30 - 20,slCardHeight,slPickAnyOne,"Deck");
		this.deckPile.cardXStagger = 12;
	}
	this.newGame = function() {
		slShuffleDeck();
		slClearAllPiles();
		for (var i=0;i<this.tableauxPiles.length;i++) {
			if (this.variantName == "Flower Garden 66") {
				this.dealToPile(6, this.tableauxPiles[i], true);
			} else {
				this.dealToPile(5, this.tableauxPiles[i], true);
			}
		}
		this.remainderToPile(this.deckPile, true);
	}
	this.ruleGoal = "Build up the foundations from ace to king, by picking your choice of cards from the deck to play to the tableaux or foundation.";
	this.ruleVariants = {"Flower Garden 66" : "Same game play, but tableux is 6 columns of 6 cards each, making it more difficult"};
	this.ruleStats = "mesm";
}
flowerGardenGame.name = "Flower Garden";
flowerGardenGame.variants = new Array("Flower Garden 66");
AddGame(flowerGardenGame);


function shamrocksGame(variantName) {
	this.inheritsFrom = solitaireGame;
	this.inheritsFrom(variantName);
	this.buildPiles = function() {
		this.buildVertTableaux(20,slCardTableHeight - 10, 10, 3);
		this.buildVertTableaux(20,slCardTableHeight - 10, 10 + 1 * (slCardWidth + 30), 3);
		this.buildVertTableaux(20,slCardTableHeight - 10, 10+ 2 * (slCardWidth + 30), 3);
		this.buildVertTableaux(20,slCardTableHeight - 10, 10+ 3 * (slCardWidth + 30), 3);
		this.buildVertTableaux(20,slCardTableHeight - 10, 10+ 4 * (slCardWidth + 30), 3);
		this.buildVertTableaux(20,slCardTableHeight - 10, 10+ 5 * (slCardWidth + 30), 3);
		for (var i=0;i<18;i++) {
			this.tableauxPiles[i].blankPileTopCard = "--"; // can't fill empty
			this.tableauxPiles[i].dragPileRule = false; // can't drag multiple
			this.tableauxPiles[i].dropRule = slDropAnyUpOrDown;
			this.tableauxPiles[i].pileLimit = 3;
			// build horizontally
			this.tableauxPiles[i].cardXStagger = 10;
			this.tableauxPiles[i].cardYStagger = 0;
		}
		this.buildHorizFoundation(slCardTableWidth - 4 * slCardWidth - 30, slCardTableWidth - 10, 10, 4);
	}
	this.newGame = function() {
		slShuffleDeck();
		slClearAllPiles();
		for (var i=0;i<this.tableauxPiles.length;i++) {
			if (i == this.tableauxPiles.length-1) {
				this.dealToPile(1, this.tableauxPiles[i], true);
			} else {
				this.dealToPile(3, this.tableauxPiles[i], true);
			}
		}
	}
	this.ruleStats = "mmsh";
}
shamrocksGame.name = "Shamrocks";
AddGame(shamrocksGame);


function eightOffGame(variantName) {
	this.inheritsFrom = solitaireGame;
	this.inheritsFrom(variantName);
	this.cardSize = mediumCards;
	this.buildPiles = function() {
		this.buildHorizFoundation(slCardTableWidth/2 - 2 * slCardWidth - 15, slCardTableWidth - 10, 10, 4);
		this.buildHorizTableaux(slCardWidth + 10, slCardTableWidth - slCardWidth - 10, slCardHeight + 20, 8);
		for (var i=0;i<8;i++) {
			this.tableauxPiles[i].blankPileTopCard = "*X"; // can fill empty
			this.tableauxPiles[i].dropRule = slDropRuleSameSuitDown;
			this.tableauxPiles[i].dragPileRule = false; // can't drag multiple
		}
		this.buildVertPiles(10, slCardTableHeight - 20, 10, 4, 3, slFreeCellPile, "FreeCell", slCardHeight, this.freeCellPiles);
		this.buildVertPiles(10, slCardTableHeight - 20, slCardTableWidth - slCardWidth - 10, 4, 3, slFreeCellPile, "FreeCell", slCardHeight, this.freeCellPiles);
	}
	this.newGame = function() {
		slShuffleDeck();
		slClearAllPiles();
		for (var i=0;i<8;i++) {
			this.dealToPile(6, this.tableauxPiles[i], true);
		}
		this.dealToPile(1, this.freeCellPiles[0], true);
		this.dealToPile(1, this.freeCellPiles[1], true);
		this.dealToPile(1, this.freeCellPiles[4], true);
		this.dealToPile(1, this.freeCellPiles[5], true);
	}
	this.ruleStats = "lmSh";
}
eightOffGame.name = "Eight Off";
AddGame(eightOffGame);

function pyramidGame(variantName) {
	this.inheritsFrom = solitaireGame;
	this.inheritsFrom(variantName);
	this.buildPiles = function() {
		var pattern = [
		"......#......",
		".....#.#.....",
		"....#.#.#....",
		"...#.#.#.#...",
		"..#.#.#.#.#..",
		".#.#.#.#.#.#.",
		"#.#.#.#.#.#.#"];
		if (this.variantName == "Pair-amid") {
			this.buildOverlapTableaux(10, 10 + 7 * slCardHeight / 2, 10, slCardTableWidth - slCardWidth * 2 - 20, pattern, slStaggerOverlapPairsPile);
			this.discardPile = slCreatePile(10,10,slCardWidth,slCardHeight, slDiscardPile,"Waste");
			this.buildVertPiles(10, slCardTableHeight - slCardHeight - 20, slCardTableWidth - slCardWidth - 10, 4, 3, slStaggerOverlapPairsPile, "Foundation", slCardHeight, this.tableauxPiles);
			this.buildVertPiles(10, slCardTableHeight - slCardHeight - 20, slCardTableWidth - 2 * slCardWidth - 15, 4, 3, slStaggerOverlapPairsPile, "Foundation", slCardHeight, this.tableauxPiles);
		} else if (this.variantName == "Pyramid 14") {
			this.buildOverlapTableaux(10, 10 + 7 * slCardHeight / 2, 10, slCardTableWidth - slCardWidth - 20, pattern, slStaggerOverlap14Pile);
			this.discardPile = slCreatePile(10,10,slCardWidth,slCardHeight, slDiscardPile,"Waste");
		} else {
			this.buildOverlapTableaux(10, 10 + 7 * slCardHeight / 2, 10, slCardTableWidth - slCardWidth - 20, pattern, slStaggerOverlap13Pile);
			this.discardPile = slCreatePile(10,10,slCardWidth,slCardHeight, slDiscardPileKings,"Waste");
		}
		this.discardPile.cardXStagger = 0;
		this.buildBottomDeckAndWaste(1);
		// we've got a "hand" between the deck and waste
		this.handPile = slCreatePile(slCardTableWidth - 3 * slCardWidth - 16, slCardTableHeight - slCardHeight, slCardWidth,slCardHeight, slHandPile, "Hand");
		this.handPile.wastePile = this.wastePile;
		this.handPile.deckPile = this.deckPile;
		if (this.variantName == "Pair-amid") {
			this.handPile.dropRule = slDropRulePairs;
		} else if (this.variantName == "Pyramid 14") {
			this.handPile.dropRule = slDropRuleTotal14;
		} else {
			this.handPile.dropRule = slDropRuleTotal13;
		}
		this.deckPile.handPile = this.handPile;
		this.deckPile.dealLimit = 0; // no redeals
	}
	this.newGame = function() {
		slShuffleDeck();
		slClearAllPiles();
		for (var i=0;i<this.tableauxPiles.length;i++) {
			this.dealToPile(1, this.tableauxPiles[i], true);
		}
		this.remainderToDeck();
		this.deckPile.clickRoutine(null,0,0);
	}
	this.ruleGoal = "Combine pairs of cards to total 13 (Kings are discarded 'for free'), which are removed from play - remove all to win.";
	this.ruleVariants = {"Pyramid 14" : "Combine pairs of cards to total 14", "Pair-amid" : "Combine pairs of the same rank - use the extra eight tableaux spaces carefully!"};
	this.ruleStats = "meCL";
}
pyramidGame.name = "Pyramid";
pyramidGame.variants = new Array("Pyramid 14", "Pair-amid");
AddGame(pyramidGame);


function nestorGame(variantName) {
	this.inheritsFrom = solitaireGame;
	this.inheritsFrom(variantName);
	this.buildPiles = function() {
		this.discardPile = slCreatePile(10,10,slCardWidth,slCardHeight, slDiscardPile,"Waste");
		var columns = 8;
		if (this.variantName == "Matrimony") {
			columns = 7;
		}
		this.buildHorizTableaux(0, slCardTableWidth, slCardHeight + 20, columns);
		for (var i=0;i<columns;i++) {
			this.tableauxPiles[i].dropRule = slDropRulePairs;
			this.tableauxPiles[i].dragPileRule = false;
			this.tableauxPiles[i].removePairs = true;
		}
		if (this.variantName == "Matrimony") {
			this.buildHorizPiles(5, slCardTableWidth - 5, slCardTableHeight - slCardHeight, 10, 3, slReservePile, "Reserve", slCardHeight, this.reservePiles);
		} else {
			this.buildBottomDeckAndWaste(1);
			//this.deckPile = slCreatePile(slCardTableWidth - slCardWidth - 90,slCardTableHeight - slCardHeight,slCardWidth + 60,slCardHeight,slDeckPile,"Deck");
			//this.deckPile.wastePile = null;
			//this.deckPile.cardXStagger = -10; // stagger to the left
			this.deckPile.dealLimit = 0; // no redeals
			// we've got a "hand" between the deck and waste
			//this.handPile = slCreatePile(30, slCardTableHeight - slCardHeight, slCardWidth,slCardHeight, slHandPile, "Hand");
			//this.deckPile.handPile = this.handPile;
			//this.handPile.wastePile = this.discardPile;
			//this.handPile.deckPile = this.deckPile;
			//this.handPile.dropRule = slDropRulePairs;
		}
	}
	this.newGame = function() {
		slShuffleDeck();
		slClearAllPiles();
		for (var i=0;i<this.tableauxPiles.length;i++) {
			noSameRank = function(cardValue, pile) {
				for (var i=0;i<pile.length;i++) {
					if (slCardPip(cardValue) == slCardPip(pile[i].cardValue)) {
						return false;
					}
				}
				return true;
			}
			this.dealToPile(6, this.tableauxPiles[i], true, noSameRank);
		}
		if (this.variantName == "Matrimony") {
			for (var i=0;i<this.reservePiles.length;i++) {
				this.dealToPile(1, this.reservePiles[i], true);
			}
		} else {
			this.remainderToDeck();
			this.deckPile.clickRoutine(null,0,0);
		}
	}
	this.ruleGoal = "Combine pairs of cards of the same rank, which are removed from play - remove all to win.";
	this.ruleVariants = {"Matrimony" : "Same game play, but slightly different layout and all of remaining cards are placed in reserve available for play (easier to win as well)."};
	this.ruleStats = "meml";
}
nestorGame.name = "Nestor";
nestorGame.variants = new Array("Matrimony");
AddGame(nestorGame);


function sirTommyGame(variantName) {
	this.inheritsFrom = solitaireGame;
	this.inheritsFrom(variantName);
	this.buildPiles = function() {
		this.buildHorizFoundation(0, slCardTableWidth, 10, 4);
		this.buildHorizTableaux(0, slCardTableWidth, 20 + slCardHeight, 4);
		for (var i=0;i<4;i++) {
			this.tableauxPiles[i].dragPileRule = false;
			this.tableauxPiles[i].dropRule = true;
			this.tableauxPiles[i].cantDragFrom = this.tableauxPiles;
			this.foundationPiles[i].canDragTop = false;
		}
		this.deckPile = slCreatePile(10,10,slCardWidth + 60,slCardHeight,slDeckPile,"Deck");
		this.deckPile.wastePile = null;
		this.deckPile.cardXStagger = 0;
		this.deckPile.cardYStagger = 2; // stagger down
		this.deckPile.ruleDescription = function() {
			return "A single card at a time is flipped up, available for play onto the foundation or tableaux.";
		}
		// disconnect the click routine
		this.deckPile.realClickRoutine = this.deckPile.clickRoutine;
		this.deckPile.clickRoutine = function(){};
		this.deckPile.cursorRoutine = function() { return null; }
		this.deckPile.dealLimit = 0; // no redeals
		this.handPile = slCreatePile(10, slCardTableHeight - slCardHeight, slCardWidth,slCardHeight, slHandPile, "Hand");
		this.deckPile.handPile = this.handPile;
		this.handPile.deckPile = this.deckPile;
	}
	this.newGame = function() {
		slShuffleDeck();
		slClearAllPiles();
		this.remainderToDeck();
		this.deckPile.realClickRoutine(null,0,0);
	}
	this.ruleGoal = "Play cards from your hand onto the foundation or tableaux, building up the foundation from Ace to King.";
	this.ruleStats = "qeml";
}
sirTommyGame.name = "Sir Tommy";
AddGame(sirTommyGame);

function bristolGame(variantName) {
	this.inheritsFrom = solitaireGame;
	this.inheritsFrom(variantName);
	this.buildPiles = function() {
		this.buildHorizFoundation(slCardTableWidth / 2, slCardTableWidth, 10, 4);
		if (this.variantName == "Bristol Unsuited") {
			for (var i=0;i<4;i++) {
				this.foundationPiles[i].dropRule = slDropRuleUnsuitedFoundation;
			}
		}
		this.buildHorizTableaux(slCardWidth + 5, slCardTableWidth, 40 + slCardHeight, 8);
		for (var i=0;i<8;i++) {
			this.tableauxPiles[i].dropRule = slDropAnyDown;
			this.tableauxPiles[i].dragPileRule = false;
			if (this.variantName == "Bristol Unsuited") {
				this.tableauxPiles[i].blankPileTopCard = "--";
			}
		}
		this.buildHorizPiles(10 + slCardWidth, slCardTableWidth / 2, 10, 3, 3, slReservePile, "Reserve", slCardHeight, this.reservePiles);
		for (var i=0;i<3;i++) {
			this.reservePiles[i].cardYStagger = 3; // small overlap
		}
		this.deckPile = slCreatePile(10,10,slCardWidth + 60,slCardHeight,slDeckPile,"Deck");
		this.deckPile.cardYStagger = 3;
		this.deckPile.dealToPiles = this.reservePiles;
	}
	this.newGame = function() {
		slShuffleDeck();
		slClearAllPiles();
		for (var i=0;i<8;i++) {
			this.dealToPile(3, this.tableauxPiles[i], true);
			// the pile needs to be sorted such that kings are moved to the bottom of the pile
			var bottomIndex = 0; // move it to the bottom
			for (var j=1;j<3;j++) {
				if (slCardPip(this.tableauxPiles[i].pile[j].cardValue) == 13) {
					// swap j and bottom index
					var swap = this.tableauxPiles[i].pile[j].cardValue;
					this.tableauxPiles[i].pile[j].cardValue = this.tableauxPiles[i].pile[bottomIndex].cardValue;
					this.tableauxPiles[i].pile[bottomIndex].cardValue = swap;

					slRebrandCard(this.tableauxPiles[i].pile[j]);
					slRebrandCard(this.tableauxPiles[i].pile[bottomIndex]);

					bottomIndex++; // so two kings get sorted correctly
				}
			}

		}
		this.remainderToDeck();
		this.deckPile.clickRoutine(null,0,0);
	}
	this.ruleVariants = {"Bristol Unsuited" : "Foundations are built regardless of suit, but empty columns in the tableaux can't be filled."};
	this.ruleStats = "mesH";
}
bristolGame.name = "Bristol";
bristolGame.variants = new Array("Bristol Unsuited");
AddGame(bristolGame);

function fourSeasonsGame(variantName) {
	this.inheritsFrom = solitaireGame;
	this.inheritsFrom(variantName);
	this.buildPiles = function() {
		var left = 10;
		var right = slCardTableWidth - slCardWidth - 85 - 10;
		var center = (left + right) / 2;
		var top = 10;
		var bottom = slCardTableHeight - 2 * slCardHeight - 10;
		var middle = (top + bottom) / 2;
		this.foundationPiles.push(slCreatePile(left, top, slCardWidth, slCardHeight, slFoundationPile, "Foundation"));
		this.foundationPiles.push(slCreatePile(right, top, slCardWidth, slCardHeight, slFoundationPile, "Foundation"));
		this.foundationPiles.push(slCreatePile(left, bottom, slCardWidth, slCardHeight, slFoundationPile, "Foundation"));
		this.foundationPiles.push(slCreatePile(right, bottom, slCardWidth, slCardHeight, slFoundationPile, "Foundation"));
		for (var i=0;i<4;i++) {
			this.foundationPiles[i].pileLimit = 13;
			this.foundationPiles[i].cardXStagger = 5;
			this.foundationPiles[i].dropRule = slDropRuleUnbasedFoundation;
		}
		if (this.variantName == "Simplicity") {
			var row1 = top + (bottom - top) / 3;
			var row2 = top + 2 * (bottom - top) / 3;
			this.tableauxPiles.push(slCreatePile(left, row1, slCardWidth, slCardHeight, slTableauxPile, "Tableaux"));
			this.tableauxPiles.push(slCreatePile(center, row1, slCardWidth, slCardHeight, slTableauxPile, "Tableaux"));
			this.tableauxPiles.push(slCreatePile(right, row1, slCardWidth, slCardHeight, slTableauxPile, "Tableaux"));
			this.tableauxPiles.push(slCreatePile(left, row2, slCardWidth, slCardHeight, slTableauxPile, "Tableaux"));
			this.tableauxPiles.push(slCreatePile(center, row2, slCardWidth, slCardHeight, slTableauxPile, "Tableaux"));
			this.tableauxPiles.push(slCreatePile(right, row2, slCardWidth, slCardHeight, slTableauxPile, "Tableaux"));
		} else {
			this.tableauxPiles.push(slCreatePile(center, top, slCardWidth, slCardHeight, slTableauxPile, "Tableaux"));
			this.tableauxPiles.push(slCreatePile(center, middle, slCardWidth, slCardHeight, slTableauxPile, "Tableaux"));
			this.tableauxPiles.push(slCreatePile(center, bottom, slCardWidth, slCardHeight, slTableauxPile, "Tableaux"));
			this.tableauxPiles.push(slCreatePile(left, middle, slCardWidth, slCardHeight, slTableauxPile, "Tableaux"));
			this.tableauxPiles.push(slCreatePile(right, middle, slCardWidth, slCardHeight, slTableauxPile, "Tableaux"));
		}
		for (var i=0;i<this.tableauxPiles.length;i++) {
			if (this.variantName == "Simplicity") {
				this.tableauxPiles[i].dropRule = slDropAltDownWrap;
			} else {
				this.tableauxPiles[i].dropRule = slDropAnyDownWrap;
			}
			this.tableauxPiles[i].dragPileRule = null;
			this.tableauxPiles[i].blankPileTopCard = "*X";
			this.tableauxPiles[i].cardXStagger = 5;
			this.tableauxPiles[i].cardYStagger = 0;
		}
		this.buildBottomDeckAndWaste(1);
		// we've got a "hand" between the deck and waste
		this.deckPile.cardXStagger = -2;
		this.wastePile.cardXStagger = 5;
		this.handPile = slCreatePile(slCardTableWidth - 3 * slCardWidth - 41, slCardTableHeight - slCardHeight, slCardWidth,slCardHeight, slHandPile, "Hand");
		this.handPile.wastePile = this.wastePile;
		this.handPile.deckPile = this.deckPile;
		this.deckPile.handPile = this.handPile;
		this.deckPile.dealLimit = 0;
	}
	this.newGame = function() {
		slShuffleDeck();
		slClearAllPiles();
		for (var i=0;i<this.tableauxPiles.length;i++) {
			this.dealToPile(1, this.tableauxPiles[i], true);
		}
		// and next card to corner
		this.dealToPile(1, this.foundationPiles[0], true);
		slGlobalFoundationBase = slCardPip(this.foundationPiles[0].pile[0].cardValue);
		this.remainderToDeck();
		this.deckPile.clickRoutine(null,0,0);
	}
	this.ruleGoal = "Build the four corner foundations, with the first card determining the start of the foundation (and then wrapping from King to Ace)";
	this.ruleVariants = {"Simplicity" : "Six tableaux piles to use, but the tableaux needs to alternate suits (instead of any suit)."};
	this.ruleStats = "mmcl";
}
fourSeasonsGame.name = "Four Seasons";
fourSeasonsGame.variants = new Array("Simplicity");
AddGame(fourSeasonsGame);


function pyramidGolfGame(variantName)	{
	this.inheritsFrom = solitaireGame;
	this.inheritsFrom(variantName);
	this.holePile = null;
	this.buildPiles = function() {
		var pattern = [
		"......#......",
		".....#.#.....",
		"....#.#.#....",
		"...#.#.#.#...",
		"..#.#.#.#.#..",
		".#.#.#.#.#.#.",
		"#.#.#.#.#.#.#"];
		this.buildOverlapTableaux(10, 10 + 7 * slCardHeight / 2, 10, slCardTableWidth - slCardWidth * 2 - 20, pattern, slStaggerOverlapPairsPile);
		for (var i=0;i<28;i++) {
			this.tableauxPiles[i].dropRule = false;
			this.tableauxPiles[i].showsEmptyImage = false;
		}

		this.deckPile = slCreatePile(slCardWidth,slCardTableHeight - slCardHeight,slCardWidth + 60,slCardHeight,slDeckPile,"Deck");
		this.deckPile.flipToWasteCount = 1;
		this.deckPile.cardYStagger = 0;
		this.deckPile.cardXStagger = 10; // stagger to the left
		this.deckPile.dealLimit = 0;
		this.deckPile.showsEmptyImage = false;
		this.deckPile.currentEmptyImage = null;
		this.deckPile.isComplete = function() { return true; } // we're complete regardless

		this.holePile = slCreatePile(slCardTableWidth - slCardWidth * 2,slCardTableHeight - slCardHeight,slCardWidth, slCardHeight, golfHolePile,"Hole");
		if (this.variantName == "Pyramid Golf (Wrap)") {
			this.holePile.dropRule = slDropAnyUpOrDownWrap;
		}
		this.deckPile.wastePile = this.holePile; // put them here
		this.holePile.deckPile = this.deckPile;
	}
	this.newGame = function() {
		slShuffleDeck();
		slClearAllPiles();
		for (var i=0;i<this.tableauxPiles.length;i++) {
			this.dealToPile(1, this.tableauxPiles[i], true);
		}
		this.remainderToDeck();
		this.deckPile.clickRoutine(null,0,0);
	}
	this.ruleStats = "qecL";
	this.ruleGoal = "A cross between golf and pyramid - sink all the cards into the Waste pile, but play from a pyramid tableaux.";
	this.ruleVariants = {"Pyramid Golf (Wrap)" : "Same as regular Pyramid Golf, but you can wrap between Ace and Kings."};
}
pyramidGolfGame.name = "Pyramid Golf";
pyramidGolfGame.variants = new Array("Pyramid Golf (Wrap)");
AddGame(pyramidGolfGame);
